% This script simulates an aliasing effect by unsersampling.
% It generates a video (sinus_aliasing.avi) 
% and corresponding audio signal (sinus_aliasing.wav).

clear all;
close all;

video_length = 20; % video lengh in seconds
video_fps = 10; % frames per second for the video
numVideoFrames = video_length * video_fps; % total number of frames in the video

fs = 2000; % sampling frequency for the audio signal 

% in the video, a total of numVideoFrames tones with increasing frequency will be played.
% Each of the tones has the following length in seconds
singleToneLength = video_length/numVideoFrames; 

% how many samples can we generate for each tone?
nsamplesPerTone = singleToneLength*fs;

% time axis for a single tone
t = (0:nsamplesPerTone-1) / fs;

% to simulate a continuous sampling with matlab, we simply increase the
% sampling rate from the original fs to a higher one, so that no aliasing
% effects occur anymore. We raise the frequency by the following factor.
fs_factor_to_continuous = 10; 

% the simulated continuous time axis for each tone.
t_continuous= (0:nsamplesPerTone*fs_factor_to_continuous-1) / (fs*fs_factor_to_continuous);


figure;
set(gcf,'Position',[100 100 800 600]); % resize the figure and move it to a visible region on the screen
set(gcf, 'Color', [1 1 1]); % set background color of the figure to white

% open the video file to which we are going to write
videofile = 'sinus_aliasing.avi';
% COMPRESSION - A string indicating the compressor to use.  On UNIX, this
%     value must be 'None'.  
%     On Windows we suggest to use 'Indeo5'.
aviobj = avifile(videofile,'compression','Indeo5','quality',90,'fps',video_fps);

% we draw only an excerpt of the signal.
% Here, we can control how many samples we draw and plot from the signal
num_plot_samples = 30; 

% totalAudioSignal will contain the concatenated isolated tones with
% increasing frequency so that is sounds like a chirp signal.
totalAudioSignal = [];

% for the generated sin functions we use the following frequency range.
frequencyRange = linspace(0, fs, numVideoFrames+1);
frequencyRange(1) = [];
frequencyRange(end) = [];

for f_signal = frequencyRange
    
    % sample the sin signal
    y = sin(2*pi*(f_signal)*t);
    y_continuous = sin(2*pi*(f_signal)*t_continuous);
    totalAudioSignal = [totalAudioSignal y];
    
    % draw everyting.
    subplot(2,2,1);
    plot(t(1:num_plot_samples),y(1:num_plot_samples), 'o');
    hold on;
    plot(t_continuous(1:num_plot_samples*fs_factor_to_continuous),y_continuous(1:num_plot_samples*fs_factor_to_continuous),'-k', 'linewidth', 2);
    set(gca,'ylim',[-1 1]);
    hold off;
    title(sprintf('y(t) = sin(2*pi*t*%g), sampled at %d Hz',f_signal, fs));
    xlabel('t [s]');
    ylabel('y(t)');
    
    subplot(2,2,4);
    Y = abs(fft(y));
    plot((1:ceil(nsamplesPerTone/2))/singleToneLength,Y(1:ceil(nsamplesPerTone/2)));
    set(gca,'ylim',[0 100]);
    title(['Spectrum of y, sampled at ' num2str(fs) ' Hz']);
    xlabel('f [Hz]');
    ylabel('Intensity');
    
    interpolated_samples = sinc_interpolate(y(1:num_plot_samples),fs,fs*fs_factor_to_continuous);
    subplot(2,2,2);
    plot(t(1:num_plot_samples),y(1:num_plot_samples), 'o');
    hold on;    
    plot(t_continuous(1:length(interpolated_samples)),interpolated_samples,'-k', 'linewidth', 2);
    set(gca,'ylim',[-1 1]);
    hold off;
    title('Reconstruction of y from its samples using Shannon''s theorem');
    xlabel('t [s]');
    ylabel('y_2(t)');
    
    subplot(2,2,3);
    Ykont = abs(fft(y_continuous));
    plot((1:ceil(nsamplesPerTone*fs_factor_to_continuous/2))/singleToneLength,Ykont(1:ceil(nsamplesPerTone*fs_factor_to_continuous/2)));
    set(gca,'ylim',[0 1000]);
    title(['Spectrum of y, sampled at ' num2str(fs*fs_factor_to_continuous) ' Hz']);
    xlabel('f [Hz]');
    ylabel('Intensity');
    
    
    % after the frame has been rendered on screen, we can capture the
    % generated image using getframe.
    frame = getframe(gcf);
    % then, the captured frame can be stuffed into the video with addframe.
    aviobj = addframe(aviobj,frame);
end;

% important: close the video file after we have finished adding frames.
aviobj = close(aviobj);
disp(['Wrote output video file ' videofile '.']);

% export the wav signal that is synchronized to the video.
wavwrite(totalAudioSignal,fs,'sinus_aliasing.wav');
